﻿using CarRepairShopContracts.BindingModels;
using CarRepairShopContracts.BusinessLogicsContracts;
using CarRepairShopContracts.SearchModels;
using CarRepairShopContracts.StoragesContracts;
using CarRepairShopContracts.ViewModels;
using CarRepairShopDataModels.Models;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CarRepairShopBusinessLogic.BusinessLogics
{
    public class ShopLogic : IShopLogic
    {
        private readonly ILogger _logger;
        private readonly IShopStorage _shopStorage;

        public ShopLogic(ILogger<ShopLogic> logger, IShopStorage shopStorage)
        {
            _logger = logger;
            _shopStorage = shopStorage;
        }

        public List<ShopViewModel>? ReadList(ShopSearchModel model)
        {
            _logger.LogInformation("ReadList. Id:{Id}", model?.Id);
            var list = model == null ? _shopStorage.GetFullList() : _shopStorage.GetFilteredList(model);
            if(list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public ShopViewModel? ReadElement(ShopSearchModel model)
        {
            if(model == null) throw new ArgumentNullException(nameof(model));
            _logger.LogInformation("ReadElement. Id:{Id}", model.Id);
            var element = _shopStorage.GetElement(model);
            if(element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public bool Create(ShopBindingModel model)
        {
            CheckModel(model);
            if(_shopStorage.Insert(model) == null)
            {
                _logger.LogWarning("Create operation failed");
                return false;
            }
            return true;
        }
        public bool Update(ShopBindingModel model)
        {
            CheckModel(model);
            if(_shopStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(ShopBindingModel model)
        {
            CheckModel(model, false);
            if(_shopStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        public bool AddRepair(ShopSearchModel model, IRepairModel repair, int count)
        {
            if(model == null) throw new ArgumentNullException(nameof(model));
            if(count <= 0) throw new ArgumentNullException("Количество добавляемых изделий должно быть больше 0", nameof(count));

            _logger.LogInformation("AddRepair. ShopName:{ShopName}. Id: {Id}", model?.ShopName, model?.Id);
            var shop = _shopStorage.GetElement(model);

            if (shop == null) return false;

            if(count > shop.MaxCountRepairs) throw new ArgumentNullException("Количество добавляемых изделий превышает максимум допустимых");

            if (!shop.ShopRepairs.ContainsKey(repair.Id))
            {
                shop.ShopRepairs[repair.Id] = (repair, count);
            }
            else
            {
                shop.ShopRepairs[repair.Id] = (repair, shop.ShopRepairs[repair.Id].Item2 + count);
            }

            _shopStorage.Update(new ShopBindingModel()
            {
                ShopName = shop.ShopName,
                ShopAddress = shop.ShopAddress,
                DateOpen = shop.DateOpen,
                MaxCountRepairs = shop.MaxCountRepairs,
                ShopRepairs = shop.ShopRepairs
            });
            return true;
        }

        private void CheckModel(ShopBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.ShopName))
            {
                throw new ArgumentNullException("Нет названия магазина", nameof(model.ShopName));
            }
            if (string.IsNullOrEmpty(model.ShopAddress))
            {
                throw new ArgumentNullException("Нету адреса магазина", nameof(model.ShopAddress));
            }
            _logger.LogInformation("Repair. ShopName:{ShopName}. ShopAddress:{ShopAddress}. Id:{Id}", model.ShopName, model.ShopAddress, model.Id);
            var element = _shopStorage.GetElement(new ShopSearchModel
            {
                ShopName = model.ShopName
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Магазин с таким названием уже есть");
            }
        }

        public bool AddRepairs(IRepairModel model, int count)
        {
            if(count <= 0)
            {
                throw new ArgumentNullException("Количество должно быть больше 0", nameof(count));
            }
            _logger.LogInformation("AddRepairs. Repair: {Repair}. Count: {Count}", model?.RepairName, count);

            var capacity = _shopStorage.GetFullList().Select(x => x.MaxCountRepairs - x.ShopRepairs.Select(x => x.Value.Item2).Sum()).Sum() - count;

            if(capacity < 0)
            {
                _logger.LogWarning("AddRepairs operation failed. Sell {count} Repairs ", capacity);
                return false;
            }

            foreach(var shop in _shopStorage.GetFullList())
            {
                if(shop.MaxCountRepairs - shop.ShopRepairs.Select(x => x.Value.Item2).Sum() < count)
                {
                    if(!AddRepair(new() { Id = shop.Id }, model, shop.MaxCountRepairs - shop.ShopRepairs.Select(x => x.Value.Item2).Sum())){
                        _logger.LogWarning("AddRepairs operation failed.");
                        return false;
                    }
                    count -= shop.MaxCountRepairs - shop.ShopRepairs.Select(x => x.Value.Item2).Sum();
                }
                else
                {
                    if (!AddRepair(new() { Id = shop.Id}, model, count))
                    {
                        _logger.LogWarning("AddRepairs operation failed.");
                        return false;
                    }
                    count -= count;
                }
                if (count == 0)
                {
                    return true;
                }
            }
            return true;
        }
        
        public bool SellRepairs(IRepairModel model, int count)
        {
            return _shopStorage.SellRepairs(model, count);
        }
    }
}
